@@ -325,7 +325,6 @@ class EmDocument(object):
   __getitem__ = __getattr__
   __delitem__ = __delattr__
 
-
 class DocumentMetaclass(EmDocumentMetaclass):
   def __new__(cls, clsname, parents, attrs):
     attrs["_write_batch"] = WriteBatch()
@@ -461,6 +460,9 @@ class Document(EmDocument):
     if not cls.indexdb:
       raise DatabaseError("indexdb is not defined for `{0}`".format(cls.__name__))
 
+    if field in ("$key", "$bucket"):
+      return
+
     if not (field and field in cls._meta and cls._meta[field]._index):
       raise DatabaseError("Field '{0}' is not indexed!".format(field))
 
@@ -483,6 +485,11 @@ class Document(EmDocument):
     """
     cls._ensure_indexdb_exists(field)
 
+    if field == "$bucket":
+      return [key for key, _ in cls.db.RangeIter()]
+    if field == "$key":
+      return [key for key, _ in cls.db.RangeIter(start_value, end_value)]
+
     if isinstance(cls._meta[field], NumberProperty):
       start_value = float(start_value)
       if end_value is not None:
@@ -517,27 +524,32 @@ class Document(EmDocument):
     """
     cls._ensure_indexdb_exists(field)
 
-    if isinstance(cls._meta[field], NumberProperty):
-      start_value = float(start_value)
-      if end_value is not None:
-        end_value = float(end_value)
-
-    if end_value is None:
-      try:
-        keys = json.loads(cls._get_indexdb().Get(_INDEX_KEY.format(f=field, v=start_value)))
-      except KeyError:
-        keys = []
-
-      for key in keys:
+    if field == "$bucket":
+      for key, _ in cls.db.RangeIter():
+        yield cls(key).reload()
+    elif field == "$key":
+      for key, _ in cls.db.RangeIter(start_value, end_value):
         yield cls(key).reload()
-
     else:
-      for index_value, keys in cls._get_indexdb().RangeIter(_INDEX_KEY.format(f=field, v=start_value), _INDEX_KEY.format(f=field, v=end_value)):
-        keys = json.loads(keys)
+      if isinstance(cls._meta[field], NumberProperty):
+        start_value = float(start_value)
+        if end_value is not None:
+          end_value = float(end_value)
+
+      if end_value is None:
+        try:
+          keys = json.loads(cls._get_indexdb().Get(_INDEX_KEY.format(f=field, v=start_value)))
+        except KeyError:
+          keys = []
+
         for key in keys:
           yield cls(key).reload()
 
-
+      else:
+        for index_value, keys in cls._get_indexdb().RangeIter(_INDEX_KEY.format(f=field, v=start_value), _INDEX_KEY.format(f=field, v=end_value)):
+          keys = json.loads(keys)
+          for key in keys:
+            yield cls(key).reload()
 
   def clear(self, to_default=True):
     EmDocument.clear(self, to_default)
@@ -772,3 +784,19 @@ class Document(EmDocument):
     else:
       db = cls._get_db(db)
       db.Delete(cls.key, sync)
+
+  def __eq__(self, other):
+    """Check equality. However, this only checks if the key are the same and
+    not the content. If the content is different and the key is the same this
+    will return True.
+
+    Args:
+      other: The other document
+
+    Returns:
+      True if the two document's key are the same, False otherwise.
+    """
+    if isinstance(other, Document):
+      return self.key == other.key
+
+    return False
